#include "config.h"

#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <cmocka.h>

#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <grp.h>

#define ARRAY_SIZE(a) (sizeof(a)/sizeof(a[0]))

static void test_uwrap_setegid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setegid(0);
	assert_int_equal(rc, 0);

	u = getegid();
#ifdef SYS_getegid /* Not available on Solaris */
	assert_int_equal(u, syscall(SYS_getegid));
#endif

	rc = setegid(42);
	assert_int_equal(rc, 0);

	u = getegid();
	assert_int_equal(u, 42);
}

#ifdef HAVE_SETREGID
static void test_uwrap_setregid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setregid(1, 2);
	assert_int_equal(rc, 0);

	u = getgid();
	assert_int_equal(u, 1);

	u = getegid();
	assert_int_equal(u, 2);
}
#endif

#ifdef HAVE_SETRESGID
static void test_uwrap_setresgid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setresgid(1, 2, -1);
	assert_int_equal(rc, 0);

	u = getgid();
	assert_int_equal(u, 1);

	u = getegid();
	assert_int_equal(u, 2);
}
#endif

static void test_uwrap_setgid(void **state)
{
	int rc;
	gid_t u;

	(void) state; /* unused */

	rc = setgid(-1);
	assert_int_equal(rc, -1);

	rc = setgid(42);
	assert_int_equal(rc, 0);

	u = getgid();
	assert_int_equal(u, 42);
}

static void test_uwrap_getgroups(void **state)
{
	gid_t rlist[20] = {0};
	int num_groups;
	int rc;

	(void) state; /* unused */
	num_groups = getgroups(0, NULL);
	assert_int_not_equal(num_groups, -1);

	rc = getgroups(ARRAY_SIZE(rlist), rlist);
	assert_int_equal(rc, num_groups);
	assert_int_equal(rlist[0], getegid());
}

static void test_uwrap_setgroups(void **state)
{
	gid_t glist[] = { 100, 200, 300, 400, 500 };
	gid_t rlist[16];
	int rc;

	(void) state; /* unused */

	rc = setgroups(ARRAY_SIZE(glist), glist);
	assert_int_equal(rc, 0);

	rc = getgroups(ARRAY_SIZE(rlist), rlist);
	assert_int_equal(rc, 5);

	assert_memory_equal(glist, rlist, sizeof(glist));

	/* Drop all supplementary groups. This is often done by daemons */
	memset(rlist, 0, sizeof(rlist));

	rc = setgroups(0, NULL);
	assert_int_equal(rc, 0);

	rc = getgroups(ARRAY_SIZE(rlist), rlist);
	assert_int_equal(rc, 0);

	assert_int_equal(rlist[0], 0);
}

int main(void) {
	int rc;

	const struct CMUnitTest uwrap_tests[] = {
		cmocka_unit_test(test_uwrap_getgroups),

		cmocka_unit_test(test_uwrap_setegid),
#ifdef HAVE_SETREGID
		cmocka_unit_test(test_uwrap_setregid),
#endif
#ifdef HAVE_SETRESGID
		cmocka_unit_test(test_uwrap_setresgid),
#endif
		cmocka_unit_test(test_uwrap_setgid),
		cmocka_unit_test(test_uwrap_setgroups),
	};

	rc = cmocka_run_group_tests(uwrap_tests, NULL, NULL);

	return rc;
}
