#include "config.h"

#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <cmocka.h>

#include <errno.h>
#include <sys/types.h>
#include <unistd.h>

#include <pwd.h>

#ifdef HAVE_SYS_SYSCALL_H
#include <sys/syscall.h>
#endif
#ifdef HAVE_SYSCALL_H
#include <syscall.h>
#endif

static void test_uwrap_syscall_setuid32(void **state)
{
	long int rc;
	uid_t u;

	(void) state; /* unused */

	rc = syscall(SYS_setuid32, 1);
	assert_int_equal(rc, 0);

	u = getuid();
	assert_int_equal(u, 1);
	assert_int_equal(u, syscall(SYS_getuid32));
}

static void test_uwrap_syscall_setreuid32(void **state)
{
	long int rc;
	uid_t u;

	(void) state; /* unused */

	rc = syscall(SYS_setreuid32, 2, 0);
	assert_int_equal(rc, 0);

	u = geteuid();
	assert_int_equal(u, syscall(SYS_geteuid32));

	rc = syscall(SYS_setreuid32, -1, 42);
	assert_int_equal(rc, 0);

	u = geteuid();
	assert_int_equal(u, 42);

	u = getuid();
	assert_int_equal(u, 2);
}

static void test_uwrap_syscall_setresuid32(void **state)
{
	long int rc;
	uid_t u;
	uid_t u_r, u_e, u_s;

	(void) state; /* unused */

	rc = syscall(SYS_setresuid32, 3, 0, 1);
	assert_int_equal(rc, 0);

	rc = syscall(SYS_getresuid32, &u_r, &u_e, &u_s);
	assert_return_code(rc, errno);

	assert_int_equal(u_r, 3);

	u = geteuid();
	assert_int_equal(u_e, u);

	assert_int_equal(u_s, 1);

	rc = syscall(SYS_setresuid32, -1, 42, -1);
	assert_return_code(rc, errno);

	u = getuid();
	assert_int_equal(u_r, 3);
	u = geteuid();
	assert_int_equal(u, 42);
}

int main(void) {
	int rc;

	const struct CMUnitTest uwrap_tests[] = {
		cmocka_unit_test(test_uwrap_syscall_setuid32),
		cmocka_unit_test(test_uwrap_syscall_setreuid32),
		cmocka_unit_test(test_uwrap_syscall_setresuid32),
	};

	rc = cmocka_run_group_tests(uwrap_tests, NULL, NULL);

	return rc;
}
