#include "config.h"

#include <stdarg.h>
#include <stddef.h>
#include <setjmp.h>
#include <cmocka.h>

#include <sys/types.h>
#include <unistd.h>

#include <pwd.h>

static void test_uwrap_setuid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setuid(-1);
	assert_int_equal(rc, -1);

	rc = setuid(42);
	assert_int_equal(rc, 0);

	u = getuid();
	assert_int_equal(u, 42);
}

static void test_uwrap_seteuid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = seteuid(-1);
	assert_int_equal(rc, -1);

	rc = seteuid(0);
	assert_int_equal(rc, 0);

	u = geteuid();
	assert_int_equal(u, 0);

	rc = setuid(42);
	assert_int_equal(rc, 0);

	u = getuid();
	assert_int_equal(u, 42);
}

#ifdef HAVE_SETREUID
static void test_uwrap_setreuid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setreuid(1, 2);
	assert_int_equal(rc, 0);

	u = getuid();
	assert_int_equal(u, 1);

	u = geteuid();
	assert_int_equal(u, 2);
}
#endif

#ifdef HAVE_SETRESUID
static void test_uwrap_setresuid(void **state)
{
	int rc;
	uid_t u;

	(void) state; /* unused */

	rc = setresuid(1, 2, -1);
	assert_int_equal(rc, 0);

	u = getuid();
	assert_int_equal(u, 1);

	u = geteuid();
	assert_int_equal(u, 2);
}
#endif

#ifdef HAVE_GETRESUID
static void test_uwrap_getresuid(void **state)
{
	int rc;
	uid_t ru, eu, su;

	(void) state; /* unused */

	rc = setresuid(1, 2, -1);
	assert_int_equal(rc, 0);

	ru = getuid();
	assert_int_equal(ru, 1);

	eu = geteuid();
	assert_int_equal(eu, 2);

	rc = getresuid(&ru, &eu, &su);
	assert_int_equal(ru, 1);
	assert_int_equal(eu, 2);
}
#endif

int main(void) {
	int rc;

	const struct CMUnitTest uwrap_tests[] = {
		cmocka_unit_test(test_uwrap_setuid),
		cmocka_unit_test(test_uwrap_seteuid),
#ifdef HAVE_SETREUID
		cmocka_unit_test(test_uwrap_setreuid),
#endif
#ifdef HAVE_SETRESUID
		cmocka_unit_test(test_uwrap_setresuid),
#endif
#ifdef HAVE_GETRESUID
		cmocka_unit_test(test_uwrap_getresuid),
#endif
	};

	rc = cmocka_run_group_tests(uwrap_tests, NULL, NULL);

	return rc;
}
